#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from datetime import datetime

from elasticsearch_dsl import Search, Q
from pyramid.threadlocal import get_current_registry
from pyramid_es import get_client

from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.view.interfaces import IWfView, IViewQuery, IViewQueryEsParamsExtension, \
    IViewQueryFilterExtension
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.list import unique_iter
from pyams_utils.request import check_request
from pyams_utils.timezone import tztime
from pyams_workflow.interfaces import IWorkflow


@adapter_config(name='es', context=IWfView, provides=IViewQuery)
class EsViewQuery(ContextAdapter):
    """View query for Elasticsearch"""

    def get_es_params(self, context):
        view = self.context
        registry = get_current_registry()
        # check publication dates
        now = tztime(datetime.utcnow())
        params = Q('range', **{'workflow.effective_date': {'lte': now}})
        params &= Q('bool', must=Q('range', **{'workflow.push_end_date': {'gte': now}})) | \
                  Q('bool', must_not=Q('exists', **{'field': 'workflow.push_end_date'}))
        # check content types
        content_types = view.get_content_types(context)
        if content_types:
            params &= Q('terms', **{'content_type': content_types})
        # check data types
        data_types = view.get_data_types(context)
        if data_types:
            params &= Q('terms', **{'data_type': data_types})
        # check workflow states
        wf_params = []
        for workflow in registry.getAllUtilitiesRegisteredFor(IWorkflow):
            wf_params.extend(workflow.published_states)
        params &= Q('terms', **{'workflow.status': wf_params})
        # check custom extensions
        for name, adapter in sorted(registry.getAdapters((view,), IViewQueryEsParamsExtension),
                                    key=lambda x: x[1].weight):
            new_params = adapter.get_es_params(context)
            if new_params:
                params &= new_params
        return params

    def get_results(self, context, sort_index, reverse, limit):
        request = check_request()
        registry = request.registry
        client = get_client(request)
        params = self.get_es_params(context)
        search = Search(using=client.es, index=client.index) \
            .query(params) \
            .sort({
                'workflow.{0}'.format(sort_index): {
                    'order': 'desc' if reverse else 'asc',
                    'unmapped_type': 'date'
                }
            }) \
            .source(['internal_id'])
        if limit:
            search = search[:limit]
        items = CatalogResultSet([result.internal_id for result in search])
        for name, adapter in sorted(registry.getAdapters((self.context,), IViewQueryFilterExtension),
                                    key=lambda x: x[1].weight):
            items = adapter.filter(context, items)
        return unique_iter(items)
